/* 
  Copyright (c) 2011 Stephen Chamberlin.  All rights reserved.
  Backcountry Logger by Stephen Chamberlin is licensed under a 
  Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License. (CC BY-NC-SA 3.0)
  The terms of the license may be viewed at http://creativecommons.org/licenses/by-nc-sa/3.0/
  Based on a work at www.bigmessowires.com.
  Permissions beyond the scope of this license may be available at www.bigmessowires.com.
*/

#include <avr/io.h>
#include <util/delay.h>

long readVcc() 
{   
	PRR &= ~(1<<PRADC); // power ADC
	ADCSRA |= (1<<ADEN); // enable ADC
	
	long result;   
	
#ifdef LOGGER_CLASSIC	
	// Read 1.1V reference against AVcc   
	ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);   
	_delay_ms(2); // Wait for Vref to settle   
	ADCSRA |= _BV(ADSC); // Convert   
	while (bit_is_set(ADCSRA,ADSC));   
	result = ADCL;   
	result |= ADCH<<8;   
	result = 1126400L / result; // Back-calculate AVcc in mV   
#endif

#ifdef LOGGER_MINI
	// Read ADC6 reference against AVcc   
	ADMUX = _BV(REFS0) | _BV(MUX2) | _BV(MUX1);   
	_delay_ms(2); // Wait for Vref to settle   
	ADCSRA |= _BV(ADSC); // Convert   
	while (bit_is_set(ADCSRA,ADSC));   
	result = ADCL;   
	result |= ADCH<<8;   
	result = result * 3260 / 1023; // convert to mV  	
#endif
	
	ADCSRA &= ~(1<<ADEN); // disable ADC
	PRR |= (1<<PRADC); // unpower ADC
		
	return result; 
} 

long readTemp() 
{     
	PRR &= ~(1<<PRADC); // power ADC
	ADCSRA |= (1<<ADEN); // enable ADC
	
	long result; 	
	// Read temperature sensor against 1.1V reference   
	ADMUX = _BV(REFS1) | _BV(REFS0) | _BV(MUX3);   
	_delay_ms(2); // Wait for Vref to settle   
	ADCSRA |= _BV(ADSC); // Convert   
	while (bit_is_set(ADCSRA,ADSC));   
	result = ADCL;   
	result |= ADCH<<8;   
	result = (result - 125) * 1075;   
	
	ADCSRA &= ~(1<<ADEN); // disable ADC
	PRR |= (1<<PRADC); // unpower ADC
		
	return result; 
} 
